-- 007: Cross-project issue board — add tags to projects
ALTER TABLE projects ADD COLUMN IF NOT EXISTS tags TEXT[] NOT NULL DEFAULT '{}';
CREATE INDEX IF NOT EXISTS idx_projects_tags ON projects USING GIN (tags);
